/****************************************************************************
 *                                                                          *
 *  COPYRIGHT (c) 1994 BY                                                   *
 *  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  *
 *  ALL RIGHTS RESERVED.                                                    *
 *                                                                          *
 *  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   *
 *  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   *
 *  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY  OTHER   *
 *  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   *
 *  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   *
 *  TRANSFERRED.                                                            *
 *                                                                          *
 *  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   *
 *  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   *
 *  CORPORATION.                                                            *
 *                                                                          *
 *  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   *
 *  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
 *                                                                          *
 ****************************************************************************
 *              Copyright 1994 Digital Equipment Corporation
 *                         All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
 * free right and license under any changes, enhancements or extensions 
 * made to the core functions of the software, including but not limited to 
 * those affording compatibility with other hardware or software 
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to Digital any
 * such changes, enhancements or extensions that they make and inform Digital
 * of noteworthy uses of this software.  Correspondence should be provided
 * to Digital at:
 *
 *                       Director of Licensing
 *                       Western Research Laboratory
 *                       Digital Equipment Corporation
 *                       250 University Ave
 *                       Palo Alto, California  94301
 *
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "platform.h"

#define MAXFD 256
#define STRLEN 100

struct FileStatsType
{
  char openFlag;
  char name[STRLEN];
  int type;
  int oflag;
  int reads;
  int writes;
  int lseeks;
  int fcntls;
  int bytesRead;
  int bytesWritten;
  long timeReads;
  long timeWrites;
} fdStats[MAXFD];

unsigned int ccLast = 0;

FILE *file;

void GetPlatform(PlatformType *platform)
{
  FILE *filePlatform;
  char model[100];
  int i;

  filePlatform = popen("/usr/sbin/sizer -c","r");
  assert(filePlatform != NULL);
  fscanf(filePlatform,"cpu \"%[^\"]s",model);
  for (i = 0; strcmp(model,platforms[i].model)!=0 &&
              platforms[i].model[0] != '\0'; i++);
  if (platforms[i].model[0] == '\0')
    {
      fprintf(file,"Model %s not found\n",model);  
      i = 0;
    }
  fprintf(file,"Using model %s platform parameters\n\n",platforms[i].model);
  *platform = platforms[i];
  fclose(filePlatform);
}

extern int errno;
FILE *OpenUnique(char *fileName, char *type)
{
  FILE *file;
  char name[STRLEN];

  if (getenv("ATOMUNIQUE") != NULL)
    sprintf(name,"%s.%d",fileName,getpid());
  else
    strcpy(name,fileName);

  file = fopen(name,type);
  if (file == NULL)
    {
      fprintf(stderr,"Atom: can't open %s for %s: %d\n",name, type, errno);
      exit(1);
    }
  return(file);
}

PlatformType platform;

void OpenFile()
{

  file = OpenUnique("io2.out","w");
  strcpy(fdStats[0].name,"stdin");
  fdStats[0].openFlag = 1;
  strcpy(fdStats[1].name,"stdout");
  fdStats[1].openFlag = 1;
  strcpy(fdStats[2].name,"stderr");
  fdStats[2].openFlag = 1;
  GetPlatform(&platform);
  fprintf(file, "%10s %10s %10s %10s %12s %12s %12s %12s %s\n",
	  "reads","writes","lseeks","fcntls",
	  "read(bytes)","write(bytes)","read(bps)",
          "write(bps)","File");
}

void PrintResults(long fd)
{
  int i;
  float rate;

  fprintf(file,"%10ld",fdStats[fd].reads);
  fprintf(file," %10ld",fdStats[fd].writes);
  fprintf(file," %10ld",fdStats[fd].lseeks);
  fprintf(file," %10ld",fdStats[fd].fcntls);
  fprintf(file," %12ld",fdStats[fd].bytesRead);
  fprintf(file," %12ld",fdStats[fd].bytesWritten);
  rate = (fdStats[fd].timeReads == 0) ?  0.0 : fdStats[fd].bytesRead /
          ((platform.cycleTime * fdStats[fd].timeReads) / 1000000000.0);
  fprintf(file," %12.2f", rate);
  rate = (fdStats[fd].timeWrites == 0) ?  0.0 : fdStats[fd].bytesWritten /
          ((platform.cycleTime * fdStats[fd].timeWrites) / 1000000000.0);
  fprintf(file," %12.2f", rate);
  fprintf(file," %s\n",fdStats[fd].name);
}

void CloseFile()
{
  fclose(file);
}

char saveName[STRLEN];
long saveType;
void BeforeOpen(long name, long type)
{
  strncpy(saveName, (char *) name, STRLEN);
  saveType = type;
}

ZeroFileStats(int fd)
{
  int i;

  fdStats[fd].reads = 0;
  fdStats[fd].writes = 0;
  fdStats[fd].lseeks = 0;
  fdStats[fd].fcntls = 0;
  fdStats[fd].timeReads = 0;
  fdStats[fd].timeWrites = 0;
  fdStats[fd].bytesRead = 0;
  fdStats[fd].bytesWritten = 0;
}

void AfterOpen(long fd)
{
  if (fd >= MAXFD)
    {
      fprintf(stderr,"Error: MAXFD exceeded\n");
      exit(1);
    }
  if (fdStats[fd].openFlag != 0)
    {
      fprintf(stderr,"Error: Reopen of previous file: %d\n",fd);
      return;
    }
  ZeroFileStats(fd);
  (void) strcpy(fdStats[fd].name,saveName,STRLEN);
  fdStats[fd].type = saveType;
  fdStats[fd].openFlag = 1;
}

void BeforeClose(long fd)
{
  if (fdStats[fd].openFlag == 0)
    {
      return;
    }
  PrintResults(fd);
  ZeroFileStats(fd);
  fdStats[fd].openFlag = 0;
}

int lastFd;

void BeforeWrite(long fd, long address, long size, long cc)
{
  fdStats[fd].writes++;
  fdStats[fd].bytesWritten += size;

  ccLast = cc;
  lastFd = fd;
}

void AfterWrite(long cc)
{
  unsigned int ccInt;

  ccInt = cc;
  fdStats[lastFd].timeWrites += ccInt - ccLast;
}

void BeforeRead(long fd, long address, long size, long cc)
{
  fdStats[fd].reads++;
  fdStats[fd].bytesRead += size;

  ccLast = cc;
  lastFd = fd;
}

void AfterRead(long cc)
{
  int ccInt;

  ccInt = cc;
  fdStats[lastFd].timeReads += ccInt - ccLast;
}

void BeforeSeek(long fd)
{
  fdStats[fd].lseeks++;
}

void BeforeFcntl(long fd)
{
  fdStats[fd].fcntls++;
}
