/****************************************************************************
 *                                                                          *
 *  COPYRIGHT (c) 1994 BY                                                   *
 *  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  *
 *  ALL RIGHTS RESERVED.                                                    *
 *                                                                          *
 *  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   *
 *  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   *
 *  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY  OTHER   *
 *  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   *
 *  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   *
 *  TRANSFERRED.                                                            *
 *                                                                          *
 *  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   *
 *  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   *
 *  CORPORATION.                                                            *
 *                                                                          *
 *  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   *
 *  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
 *                                                                          *
 ****************************************************************************
 *              Copyright 1994 Digital Equipment Corporation
 *                         All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
 * free right and license under any changes, enhancements or extensions 
 * made to the core functions of the software, including but not limited to 
 * those affording compatibility with other hardware or software 
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to Digital any
 * such changes, enhancements or extensions that they make and inform Digital
 * of noteworthy uses of this software.  Correspondence should be provided
 * to Digital at:
 *
 *                       Director of Licensing
 *                       Western Research Laboratory
 *                       Digital Equipment Corporation
 *                       250 University Ave
 *                       Palo Alto, California  94301
 *
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "lmemsys.h"
#include "platform.h"

struct Tag {
  long tag;
  long valid;
} icache[PCACHE_MAX / LINE_MIN],
  dcache[PCACHE_MAX / LINE_MIN];

struct BoardTag {
  long tag;
  long valid;
  int dirty;
  int number; 
} bcache[BCACHE_MAX / LINE_MIN];

struct WriteBufferType {
  long tag;
  long address;
  int valid;
} writeBuffer[WB_MAX];

struct ResultType *results;

FILE *file;

void GetPlatform(PlatformType *platform)
{
  FILE *filePlatform;
  char model[100];
  int i;

  filePlatform = popen("/usr/sbin/sizer -c","r");
  assert(filePlatform != NULL);
  fscanf(filePlatform,"cpu \"%[^\"]s",model);
  for (i = 0; strcmp(model,platforms[i].model)!=0 &&
              platforms[i].model[0] != '\0'; i++);
  if (platforms[i].model[0] == '\0')
    {
      fprintf(file,"Model %s not found\n",model);
      i = 0;
    }
  fprintf(file,"Using model %s platform parameters\n\n",platforms[i].model);
  *platform = platforms[i];
  fclose(filePlatform);
}

FILE *OpenUnique(char *fileName, char *type)
{
  FILE *file;
  char Name[200];

  if (getenv("ATOMUNIQUE") != NULL)
    sprintf(Name,"%s.%d",fileName,getpid());
  else
    strcpy(Name,fileName);

  file = fopen(Name,type);
  if (file == NULL)
    {
      fprintf(stderr,"Atom: can't open %s for %s\n",Name, type);
      exit(1);
    }
  return(file);
}

PlatformType platform;
long startPC;
int reportTimes;

void OpenFile(long pc,char *data)
{
  results = (struct ResultType *) data;
  file = OpenUnique("lmemsys.out","w");
  GetPlatform(&platform);
  reportTimes = 1;
  if (getenv("MEMSYSCOUNT") != NULL)
    {
      reportTimes = 0;
    }
  startPC = pc;
}

void InstReference(int number, long address)
{
  int index;
  long tag;

  index = (address & (platform.icacheSize-1)) >> platform.icacheShift;
  tag = address >> platform.icacheShift;

  if (icache[index].valid == 0 || icache[index].tag != tag)
    {
      results[number].imisses++;
      icache[index].tag = tag;
      icache[index].valid = 1;

      index = (address & (platform.bcacheSize-1)) >> platform.bcacheShift;
      tag = address >> platform.bcacheShift;

      if (bcache[index].valid == 0 || bcache[index].tag != tag)
	{
	  results[number].bmisses++;
	  results[number].bvictims += bcache[index].dirty;
	  bcache[index].tag = tag;
	  bcache[index].valid = 1;
	  bcache[index].dirty = 0;
	}
    }
}

void LoadReference(int number, long address)
{
  int index;
  long tag;

  index = (address & (platform.dcacheSize-1)) >> platform.dcacheShift;
  tag = address >> platform.dcacheShift;

  if (dcache[index].valid == 0 || dcache[index].tag != tag)
    {
      results[number].drmisses++;
      dcache[index].tag = tag;
      dcache[index].valid = 1;

      index = (address & (platform.bcacheSize-1)) >> platform.bcacheShift;
      tag = address >> platform.bcacheShift;

      if (bcache[index].valid == 0 || bcache[index].tag != tag)
	{
	  results[number].bmisses++;
	  results[number].bvictims += bcache[index].dirty;
	  bcache[index].tag = tag;
	  bcache[index].valid = 1;
	  bcache[index].dirty = 0;
	}
    }
}

long time = 0;

void StoreReference(int number, long address)
{
  int index,i;
  long tag,bcAddress;

  tag = address >> platform.dcacheShift;
  for (i = 0; i < platform.wbSize; i++)
    {
      if (tag == writeBuffer[i].tag && writeBuffer[i].valid) break;
    }
  if (i >= platform.wbSize)
    {
      i = ++time & (platform.wbSize-1);
      bcAddress = writeBuffer[i].address;
      writeBuffer[i].tag = tag;
      writeBuffer[i].address = address;
      results[number].dwmisses++;

      if (writeBuffer[i].valid)
        {
          index = (bcAddress & (platform.bcacheSize-1)) >>
                   platform.bcacheShift;
          tag = bcAddress >> platform.bcacheShift;

          if (bcache[index].valid == 0 || bcache[index].tag != tag)
            {
              results[number].bmisses++;
              results[number].bvictims += bcache[index].dirty;
              bcache[index].tag = tag;
              bcache[index].valid = 1;
            }
          bcache[index].dirty = 1;
        }
      writeBuffer[i].valid = 1;
    }
}

double iServiceTotal,drServiceTotal,dwServiceTotal = 0.0;
double bServiceTotal,vServiceTotal,serviceTotal = 0.0;

double iServiceOther,drServiceOther,dwServiceOther = 0.0;
double bServiceOther,vServiceOther,serviceOther = 0.0;

void PrintResultTime(int numberFirst,int numberNext,char *name,char*fileName)
{
  double iService,drService,dwService,bService,vService,service;
  int i,headerFlag;

  headerFlag = 1;
  for (i = numberFirst; i < numberNext; i++)
    {
      iService = ((double) results[i].imisses * platform.iMissService) 
	         / 1000000.0;
      drService = ((double) results[i].drmisses * platform.dMissService) 
                 / 1000000.0; 
      dwService = ((double) results[i].dwmisses * platform.dWriteService) 
                 / 1000000.0; 
      bService = ((double) results[i].bmisses * platform.bMissService) 
                 / 1000000.0; 
      vService = ((double) results[i].bvictims* platform.bVictimService) 
                 / 1000000.0;
      service = iService + drService +dwService + bService + vService;

      iServiceTotal += iService;
      drServiceTotal += drService;
      dwServiceTotal += dwService;
      bServiceTotal += bService;
      vServiceTotal += vService;
      serviceTotal += service;
      
      if (service >= 0.1)
	{
	  if (headerFlag)
	    {
	      fprintf(file, "------------------------------------------------------------------------------------------------\n");
	      fprintf(file,"%s: %s\n",name, fileName);
	      fprintf(file, "------------------------------------------------------------------------------------------------\n");
	      fprintf(file,
		      "%7s %11s %12s %12s %12s %12s %12s %12s\n",
		      "Line",
		      "PC",
		      "imisses(ms)", 
		      "drmisses(ms)",
		      "dwrites(ms)",
		      "bmisses(ms)",
		      "bvictims(ms)",
		      "total(ms)");
	      headerFlag = 0;
	    }
	  fprintf(file,
	  "%7d 0x%9lx %12.1lf %12.1lf %12.1lf %12.1lf %12.1lf %12.1lf\n", 
		  results[i].line, startPC + results[i].pc,
		  iService,drService,dwService,bService,
		  vService,service);
	}
      else if (service > 0.0)
	{
	  iServiceOther += iService;
	  drServiceOther += drService;
	  dwServiceOther += dwService;
	  bServiceOther += bService;
	  vServiceOther += vService;
	  serviceOther += service;
	}
    }
}

long iCountTotal, drCountTotal, dwCountTotal, bCountTotal, vCountTotal;

void PrintResultCount(int numberFirst,int numberNext,char *name, char*fileName)
{
  long count;
  int i,headerFlag;

  headerFlag = 1;
  for (i = numberFirst; i < numberNext; i++)
    {
      count = results[i].imisses + results[i].drmisses + results[i].dwmisses +
              results[i].bmisses + results[i].bvictims;

      iCountTotal += results[i].imisses;
      drCountTotal += results[i].drmisses;
      dwCountTotal += results[i].dwmisses;
      bCountTotal += results[i].bmisses;
      vCountTotal += results[i].bvictims;
      
      if (count > 0)
	{
	  if (headerFlag)
	    {
	      fprintf(file, "------------------------------------------------------------------------------------------------\n");
	      fprintf(file,"%s: %s\n",name, fileName);
	      fprintf(file, "------------------------------------------------------------------------------------------------\n");
	      fprintf(file,
		      "%7s %11s %12s %12s %12s %12s %12s\n",
		      "Line", "PC", "imisses", "drmisses", "dwrites",
		      "bmisses", "bvictims");
	      headerFlag = 0;
	    }
	  fprintf(file,
	  "%7d 0x%9lx %12ld %12ld %12ld %12ld %12ld\n", 
		  results[i].line, startPC + results[i].pc,
		  results[i].imisses,results[i].drmisses,results[i].dwmisses,
		  results[i].bmisses, results[i].bvictims);
	}
    }
}

void PrintResult(int numberFirst, int numberNext, char *name, char*fileName)
{
  if (reportTimes)
    {
      PrintResultTime(numberFirst, numberNext, name, fileName);
    }
  else
    {
      PrintResultCount(numberFirst, numberNext, name, fileName);
    }
}

void CloseFile()
{
  if (reportTimes)
    {
      fprintf(file,
	  "\n%7s %11s %12.1lf %12.1lf %12.1lf %12.1lf %12.1lf %12.1lf\n", 
	  "Other","",
	  iServiceOther,drServiceOther,dwServiceOther,bServiceOther,
	  vServiceOther, serviceOther);
      fprintf(file,
	  "\n%7s %11s %12.1lf %12.1lf %12.1lf %12.1lf %12.1lf %12.1lf\n", 
	  "Summary","",
	  iServiceTotal,drServiceTotal,dwServiceTotal,bServiceTotal,
	  vServiceTotal, serviceTotal);
    }
  else
    {
      fprintf(file,
	  "\n%7s %11s %12ld %12ld %12ld %12ld %12ld\n", 
	  "Summary","",
	  iCountTotal,drCountTotal,dwCountTotal,bCountTotal,
	  vCountTotal);
    }
  fclose(file);
}
