/****************************************************************************
 *                                                                          *
 *  COPYRIGHT (c) 1994 BY                                                   *
 *  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  *
 *  ALL RIGHTS RESERVED.                                                    *
 *                                                                          *
 *  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   *
 *  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   *
 *  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY  OTHER   *
 *  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   *
 *  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   *
 *  TRANSFERRED.                                                            *
 *                                                                          *
 *  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   *
 *  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   *
 *  CORPORATION.                                                            *
 *                                                                          *
 *  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   *
 *  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
 *                                                                          *
 ****************************************************************************
 *              Copyright 1994 Digital Equipment Corporation
 *                         All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
 * free right and license under any changes, enhancements or extensions 
 * made to the core functions of the software, including but not limited to 
 * those affording compatibility with other hardware or software 
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to Digital any
 * such changes, enhancements or extensions that they make and inform Digital
 * of noteworthy uses of this software.  Correspondence should be provided
 * to Digital at:
 *
 *                       Director of Licensing
 *                       Western Research Laboratory
 *                       Digital Equipment Corporation
 *                       250 University Ave
 *                       Palo Alto, California  94301
 *
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */
#include <stdio.h>
#include <stdlib.h>
#include "lpipe.h"

BasicBlockData *results;
long instructions_total;
long packedcycles_total;
long cycles_total;

FILE *OpenUnique(char *fileName, char *type)
{
  FILE *file;
  char Name[200];

  if (getenv("ATOMUNIQUE") != NULL)
    sprintf(Name,"%s.%d",fileName,getpid());
  else
    strcpy(Name,fileName);

  file = fopen(Name,type);
  if (file == NULL)
    {
      fprintf(stderr,"Atom: can't open %s for %s\n",Name, type);
      exit(1);
    }
  return(file);
}

FILE *file;
void OpenFile(char *data)
{
  file = OpenUnique("pipe.dynamic","w");
  results =  (BasicBlockData *) data;
}

void CloseFile()
{
  fclose(file);
}

void BlockCount(long number)
{
  results[number].ExecutionCount++;
}

void PrintHeader(long numberBlocks) {
  long i;
  long nInstructions = 0;
  long nPackedCycles = 0;
  long nCycles = 0;

  for (i = 0; i < numberBlocks; i++) {
      nInstructions += results[i].NInstructions * results[i].ExecutionCount;
      nPackedCycles += results[i].NPackedCycles * results[i].ExecutionCount;
      nCycles += results[i].NCycles * results[i].ExecutionCount;
  }
  instructions_total += nInstructions;
  packedcycles_total += nPackedCycles;
  cycles_total += nCycles;
  fprintf(file,"%25s %13s %13s %13s %5s %13s %6s %6s\n",
	  "Procedure", "Instructions", "Dual Packed", "Cycles", "CPI", "Pipe Stall", "CPct", "SPct");
}

void PrintProc(long numberFirst, long number, char *name, char *fileName)
{
  long i;
  long nInstructions = 0;
  long nPackedCycles = 0;
  long nCycles = 0;
  float cpi;

  for (i = numberFirst; i < number; i++) {
      nInstructions += results[i].NInstructions * results[i].ExecutionCount;
      nPackedCycles += results[i].NPackedCycles * results[i].ExecutionCount;
      nCycles += results[i].NCycles * results[i].ExecutionCount;
  }
  if (nInstructions == 0) return;

  fprintf(file,"%25s %13ld %13ld %13ld %5.2f %13ld %6.2f %6.2f\n",
	  name, nInstructions,
	  nPackedCycles,
	  nCycles, nCycles/(float)nInstructions,
	  nCycles-nPackedCycles,
	  nCycles*100/(float)cycles_total,
	  (nCycles-nPackedCycles)*100/(float)cycles_total
	  );

}

void PrintTotal()
{
  fprintf(file,
  "-----------------------------------------------------------------------------------------------------\n");
  fprintf(file,
	  "%25s %13ld %13ld %13ld %5.2f %13ld        %6.2f\n", "totals",
	  instructions_total, packedcycles_total, cycles_total,
	  cycles_total/(float)instructions_total,
	  cycles_total-packedcycles_total,
	  (cycles_total-packedcycles_total)*100/(float)cycles_total
	  );
}
