/****************************************************************************
 *                                                                          *
 *  COPYRIGHT (c) 1994 BY                                                   *
 *  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  *
 *  ALL RIGHTS RESERVED.                                                    *
 *                                                                          *
 *  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   *
 *  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   *
 *  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY  OTHER   *
 *  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   *
 *  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   *
 *  TRANSFERRED.                                                            *
 *                                                                          *
 *  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   *
 *  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   *
 *  CORPORATION.                                                            *
 *                                                                          *
 *  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   *
 *  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
 *                                                                          *
 ****************************************************************************
 *              Copyright 1994 Digital Equipment Corporation
 *                         All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
 * free right and license under any changes, enhancements or extensions 
 * made to the core functions of the software, including but not limited to 
 * those affording compatibility with other hardware or software 
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to Digital any
 * such changes, enhancements or extensions that they make and inform Digital
 * of noteworthy uses of this software.  Correspondence should be provided
 * to Digital at:
 *
 *                       Director of Licensing
 *                       Western Research Laboratory
 *                       Digital Equipment Corporation
 *                       250 University Ave
 *                       Palo Alto, California  94301
 *
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

/*
 * Important Information about using this file:
 * 
 *  1. The following table contains platform-specific information for
 *     several Digital platforms.  The table below is indexed by the
 *     platform-type as reported by the /usr/sbin/sizer utility.  To
 *     discover your platform type, enter the following command:
 *
 *                   % /usr/sbin/sizer -c
 *
 *     Note that /usr/sbin/sizer returns the same "name" for a family
 *     of platforms.
 *
 *
 *  2. Not all platforms, or types of platforms, are represented.  If 
 *     your platform is not present, it may be listed at the bottom of  
 *     this file.  If so, you must replace the line with the same "name" 
 *     in the table as the one that you are moving.  See the instructions 
 *     below.
 *
 *
 *  3. In instances where the original information contained in a previous
 *     version of this file was different than information gathered to 
 *     update this file, the updated information was inserted.  However,
 *     no effort was made to challenge the integrity of the original data
 *     which is still contained in this file.  The entries DEC3000_400, 
 *     DEC3000_600 and DEC10000 were removed.  These "names" are never 
 *     returned by the /usr/sbin/sizer utility, and therefore these entries 
 *     were never used.
 *
 *
 *  4. It is important to consider that as architectures become more 
 *     complex, it becomes very difficult to quantify behaviour.  Although 
 *     these numbers have been measured in a specific context, they may not 
 *     be accurate in your context.  These numbers should be used as guide-
 *     lines only.  
 *
 *
 *  5. Note that the Add-on tools are designed only for uniprocessor systems.
 *     The behaviour of multiprocessor systems is not handled by the 
 *     Add-on tools.
 *
 *
 *  6. Note that the Add-on tools were written for the 21064, which does 
 *     not have a secondary cache.  The behaviour of the secondary cache, 
 *     and its effects, are not defined in the Add-on tools.
 *
 *
 *  7. Also note that the Add-on tools are not a supported product.
 *
 */

#ifndef PLATFORM
#define PLATFORM

#define WB_MAX            5
#define TB_MAX           32
#define LINE_MIN         32
#define PCACHE_MAX    16384
#define BCACHE_MAX  4194304
#define PAGE_SIZE      8192
#define PAGE_SHIFT       13

struct platformType {
  char  *model;        /* Model number                                   */
  int   itbSize;       /* User translation buffer entries                */
  int   dtbSize;       /* number of entries in the translation buffer    */
  float tbService;     /* Cycles to servie translation buffer miss       */
  int   wbSize;        /* number of entries in the on-chip write buff    */
  int   icacheSize;    /* Size(bytes) of the on chip instruction cache   */
  int   icacheShift;   /* log 2 of the instruction cache line size       */
  int   dcacheSize;    /* Size(bytes) of the on chip data cache          */
  int   dcacheShift;   /* log 2 of the data cache line size              */
  int   bcacheSize;    /* Size(bytes) of the board cache                 */
  int   bcacheShift;   /* log 2 of the board cache line size             */
  float cycleTime;     /* Processor cycle time in nanoseconds            */
  float iMissService;  /* Average service time of instruction cache miss */
  float dMissService;  /* Average service time of data cache miss        */
  float dWriteService; /* Average service time of data cache board write */
  float bMissService;  /* Average service time of board cache miss       */
  float bVictimService;/* Average service time of victim only            */
} platforms[] = {
/*
 *  first item acts as the default
 */            /* chip specific parameters                   */  /* platform specific parameters */
"DEC3000_500",12, 32, 50.0, 4, 8192, 5, 8192, 5, 524288, 5, 6.66,  66.6,  66.6,  99.9, 260.0, 190.0,
"DEC3000_300",12, 32, 50.0, 4, 8192, 5, 8192, 5, 262144, 5, 6.66, 106.6, 106.6, 133.3, 450.0, 400.0,
"DEC4000",    12, 32, 50.0, 4, 8192, 5, 8192, 5,1048576, 5, 6.25,  50.0,  50.0,  75.0, 182.0, 118.7,
"DEC7000",    12, 32, 50.0, 4, 8192, 5, 8192, 5,4194304, 6, 5.00,  75.0,  75.0,  75.0, 400.0,  95.0,
"DEC1000",    48, 64, 50.0, 6, 8192, 5, 8192, 5,2097152, 6, 3.76, 180.4,  48.8,  71.4, 451.2,  22.5,
"DEC1000A",   48, 64, 50.0, 6, 8192, 5, 8192, 5,2097152, 6, 3.76, 180.4,  48.8,  71.4, 451.2,  22.5,
"DEC2100_A50",12, 32, 50.0, 4, 8192, 5, 8192, 5,2097152, 5, 3.33, 196.4,  33.3,  63.2, 399.6,  33.3,
"DEC2100_A500",48,64, 50.0, 6, 8192, 5, 8192, 5,4194304, 6, 3.33, 159.8,  43.3,  63.2, 399.6,  19.9,
"",            0,  0,  0.0, 0,    0, 0,    0, 0,      0, 0,  0.0,   0.0,   0.0,   0.0,   0.0,   0.0
	   };

typedef struct platformType PlatformType;


#endif PLATFORM

/*
 *
 This is the list of additional system parameters.  To update the table
 for your system, follow these steps:

	1. Run the sizer utility on your system:

		% /usr/sbin/sizer -c

	2. This name corresponds to an entry in the table.  Remove the 
	   current entry from the table.  

	3. Get the correct entry for your system from the comment section 
	   below and insert it into the table.  Now the table is correct 
	   for your system.



   There are four different systems in the Alpha Server 1000 family:

		21164/266 MHz/2 MB	listed above
		21064a/266 MHz/2 MB
		21064a/233 MHz/2 MB
		21064/200 MHz/2 MB

Alpha Server 1000: 21164/266 MHz/2 MB 
"DEC1000",    48, 64, 50.0, 6, 8192, 5, 8192, 5,2097152, 6, 3.76, 180.4,  48.8,  71.4, 451.2,  22.5,

Alpha Server 1000: 21064a/266 MHz/2 MB
"DEC1000",     12, 32, 50.0, 4,16384, 5,16384, 5,2097152, 5, 3.76, 221.8,  37.6,  71.4, 451.2,  37.6,

Alpha Server 1000: 21064a/233 MHz/2 MB
"DEC1000",     12, 32, 50.0, 4,16384, 5,16384, 5,2097152, 5, 4.29, 253.1,  42.9,  81.5, 514.8,  42.9,

Alpha Server 1000: 21064/200 MHz/2 MB
"DEC1000",     12, 32, 50.0, 4, 8192, 5, 8192, 5,2097152, 5, 5.00, 295.0,  50.0,  95.0, 600.0,  50.0,




   There are two types of platforms on the AlphaServer 1000A family:

			21164/266 MHz/2 MB	listed above
			21064a/266 MHz/2 MB

AlphaServer 1000A: 21164/266 MHz/2 MB
"DEC1000A",   48, 64, 50.0, 6, 8192, 5, 8192, 5,2097152, 6, 3.76, 180.4,  48.8,  71.4, 451.2,  22.5,

AlphaServer 1000A: 21064a/266 MHz/2 MB
"DEC1000A",   12, 32, 50.0, 4, 8192, 5, 8192, 5,2097152, 5, 3.76, 221.8,  37.6,  71.4, 451.2,  37.6,


   There are four different platforms in the AlphaStation 400 family:

			21064a/300 MHz/2 MB    listed above
			21064a/266 MHz/2 MB
			21064a/266 MHz/512kb 
			21064a/233 MHz/512kb
			21064/166 MHz/512kb


AlphaStation 400: 21064a/300 MHz/2 MB    listed above
"DEC2100_A50",12, 32, 50.0, 4, 8192, 5, 8192, 5,2097152, 5, 3.33, 196.4,  33.3,  63.2, 399.6,  33.3,

AlphaStation 400: 21064a/266 MHz/2 MB
"DEC2100_A50",12, 32, 50.0, 4, 8192, 5, 8192, 5,2097152, 5, 3.76, 221.8,  37.6,  71.4, 451.2,  37.6,

AlphaStation 400: 21064a/266 MHz/512kb
"DEC2100_A50",12, 32, 50.0, 4, 8192, 5, 8192, 5, 525288, 5, 3.76, 221.8,  37.6,  71.4, 451.2,  37.6,

AlphaStation 400: 21064a/233 MHz/512kb
"DEC2100_A50",12, 32, 50.0, 4, 8192, 5, 8192, 5, 525288, 5, 4.29, 196.4,  42.9,  81.5, 514.8,  42.9,

AlphaStation 400: 21064/166 MHz/512kb
"DEC2100_A50",12, 32, 50.0, 4, 8192, 5, 8192, 5, 525288, 5, 6.02, 355.1,  60.2, 114.3, 722.4, 114.3,


   There are five different types of platforms in the Alpha Server 2100 family:

		21164/300 MHz/4 MB	listed above
		21164/250 MHz/4 MB
		21064a/275 MHz/4 MB
		21064a/233 MHz/1 MB
		21064/200 MHz/1 MB 


Alpha Server 210: 21164/300 MHz/4 MB	listed above
"DEC2100_A500",48,64, 50.0, 6, 8192, 5, 8192, 5,4194304, 6, 3.33, 159.8,  43.3,  63.2, 399.6,  19.9,

Alpha Server 210: 21164/250 MHz/4 MB
"DEC2100_A500",48,64, 50.0, 6, 8192, 5, 8192, 5,4194304, 6, 4.00, 192.0,  52.0,  76.0, 480.0,  24.0,

Alpha Server 210: 21064a/275 MHz/4 MB
"DEC2100_A500",12,32, 50.0, 4,16384, 5,16384, 5,4194304, 5, 3.63, 214.1,  36.3,  68.9, 435.6,  36.3,

Alpha Server 210: 21064a/233 MHz/1 MB
"DEC2100_A500",12,32, 50.0, 4,16384, 5,16384, 5,1048576, 5, 4.29, 196.4,  42.9,  81.5, 514.8,  42.9,

Alpha Server 210: 21064/200 MHz/1 MB
"DEC2100_A500",12,32, 50.0, 4, 8192, 5, 8192, 5,1048576, 5, 5.00, 295.0,  50.0,  95.0, 600.0,  50.0,

*
*/





