/****************************************************************************
 *                                                                          *
 *  COPYRIGHT (c) 1994 BY                                                   *
 *  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  *
 *  ALL RIGHTS RESERVED.                                                    *
 *                                                                          *
 *  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   *
 *  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   *
 *  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY  OTHER   *
 *  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   *
 *  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   *
 *  TRANSFERRED.                                                            *
 *                                                                          *
 *  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   *
 *  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   *
 *  CORPORATION.                                                            *
 *                                                                          *
 *  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   *
 *  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
 *                                                                          *
 ****************************************************************************
 *              Copyright 1994 Digital Equipment Corporation
 *                         All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
 * free right and license under any changes, enhancements or extensions 
 * made to the core functions of the software, including but not limited to 
 * those affording compatibility with other hardware or software 
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to Digital any
 * such changes, enhancements or extensions that they make and inform Digital
 * of noteworthy uses of this software.  Correspondence should be provided
 * to Digital at:
 *
 *                       Director of Licensing
 *                       Western Research Laboratory
 *                       Digital Equipment Corporation
 *                       250 University Ave
 *                       Palo Alto, California  94301
 *
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

 /* Note that the application must be compiled -non_shared for effective 
  * instrumentation.
  */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <sys/syscall.h>
#include "platform.h"

#define SYSCALLS LAST                           /* Last system call number */

struct SyscallType {
  long calls;
  long total;
  long context;
} syscalls[SYSCALLS];

long ccPrev;
int callPrev;
long context=0;

struct {
  int call;
  char syscall[28];
} mapSyscall[SYSCALLS];

void BeforePal(long call, long cc)
{
  if (cc >> 32 != ccPrev >> 32) context++;
  ccPrev = cc;
  callPrev = call;
}

void AfterPal(long cc)
{
  unsigned int ccInt,ccIntPrev;

  ccInt = cc;
  ccIntPrev = ccPrev;
  if (callPrev >= 0 && callPrev <= 258)
    {
      syscalls[callPrev].total += ccInt - ccIntPrev;
      syscalls[callPrev].calls++;
      if (cc >> 32 != ccPrev >> 32) syscalls[callPrev].context++;
    }
}

FILE *OpenUnique(char *fileName, char *type)
{
  FILE *file;
  char Name[200];

  if (getenv("ATOMUNIQUE") != NULL)
    sprintf(Name,"%s.%d",fileName,getpid());
  else
    strcpy(Name,fileName);

  file = fopen(Name,type);
  if (file == NULL) {
    printf("Error opening output file.\n");
    exit(1);
  }
  return(file);
}

FILE *file;

void GetPlatform(PlatformType *platform)
{
  FILE *filePlatform;
  char model[100];
  int i;

  filePlatform = popen("/usr/sbin/sizer -c","r");
  assert(filePlatform != NULL);
  fscanf(filePlatform,"cpu \"%[^\"]s",model);
  for (i = 0; strcmp(model,platforms[i].model)!=0 &&
              platforms[i].model[0] != '\0'; i++);
  if (platforms[i].model[0] == '\0')
    {
      fprintf(file,"Model %s not found\n",model);
      i = 0;
    }
  fprintf(file,"Using model %s platform parameters\n\n",platforms[i].model);
  *platform = platforms[i];
  fclose(filePlatform);
}
  
/* new routine to initialize the mapSyscall data structure.  This routine uses
 * the system include file /usr/include/sys/syscall.h to set up mapSyscall.
 * this will alleviate the need to update this file when system calls are
 * added/deleted.
 */

void InitSysCalls()
{
  FILE *fp;
  char buffer[80];
  char tmpsyscall[28];
  char tmpcall[4];
  int i=0,itmp=0;
  int done=0;

  fp = fopen("/usr/include/sys/syscall.h","r");
  assert(fp != NULL);
  while ( ! done ) {
    fscanf(fp,"%s", buffer);
    if ( ! strncmp(buffer,"#define", 7)) {
      fscanf(fp, "%s", tmpsyscall);
      if ( strncmp(tmpsyscall, "_SYS_SYSCALL_H_")) {
	/* if defining table entry, carry on*/
	fscanf(fp, "%d", &itmp);

	while (i < itmp) {
	  strcpy(mapSyscall[i].syscall,"unknown");
	  mapSyscall[i].call= i;
	  i++;
	}
	/* strip off the SYS_ during the strcpy */
	strcpy(mapSyscall[itmp].syscall,&tmpsyscall[4]);
	mapSyscall[itmp].call= itmp;
	if (i <= itmp) i++;           /* ifdefs for backwards compatibility */
      }                               /* cause repeats of numbers. this is  */
    }                                 /* the workaround.                    */
    if (i==SYSCALLS) done=1;
  }
  fclose(fp);
}

PlatformType platform;

void OpenFile()
{
  file = OpenUnique("syscall.out","w");
  GetPlatform(&platform);
  InitSysCalls();
}

void PrintSysCalls()
{
  int i;
  long total = 0;

  fprintf(file,"%25s %10s   %15s %10s\n",
	  "System Call","Calls","Time(ms)","context");

  for ( i=0; i < SYSCALLS; i++)
    {
      if (syscalls[i].total != 0)
	{
	  total += syscalls[i].total;
	  if (strcmp(mapSyscall[i].syscall,"unknown")==0)
	    {
	      fprintf(file,"%22s%3d %10ld %15.1f %10ld\n", 
		      mapSyscall[i].syscall,
		      i,syscalls[i].calls,
		      (platform.cycleTime * syscalls[i].total) / 1000000.0,
		      syscalls[i].context);
	    }
	  else
	    {
	      fprintf(file,"%25s %10ld %15.1f %10ld\n", 
                      mapSyscall[i].syscall, syscalls[i].calls,
		      (platform.cycleTime * syscalls[i].total) / 1000000.0,
		      syscalls[i].context);
	    }
	}
    }
  fprintf(file,"\n%25s %10s %15.1f %10ld\n",
	  "Total","", (platform.cycleTime * total) / 1000000.0, context);
  fclose(file);
}

